@@ -38,9 +38,9 @@ class QDFT:
         bandwidth : tuple(float, float)
             Lowest and highest frequency in hertz to be resolved.
         resolution : int, optional
-            Octave resolution, e.g. number of bins per octave.
+            Octave resolution, e.g. number of DFT bins per octave.
         latency : float, optional
-            Analysis latency factor between -1 and +1.
+            Analysis latency adjustment between -1 and +1.
         window : tuple(float, float), optional
             Cosine family window coeffs, e.g. (+0.5,-0.5) in case of hann window.
         """
@@ -59,13 +59,22 @@ class QDFT:
         twiddles = numpy.exp(+2j * numpy.pi * (quality + kernels[:, None]) / periods)
 
         inputs = numpy.zeros(periods[0], dtype=float)
-        outputs = numpy.zeros((kernels.size, size), dtype=complex)
+        outputs = numpy.zeros((size, kernels.size), dtype=complex)
+
+        dfts = numpy.zeros((0, size, kernels.size), dtype=complex)
+        dfts = self.__analyze__(dfts, inputs, outputs, periods, offsets, weights, fiddles, twiddles, window)
+        assert dfts.shape == (0, size, )
+
+        samples = numpy.zeros((0), dtype=float)
+        samples = self.__synthesize__(dfts, samples, twiddles)
+        assert samples.shape == (0, )
 
         self.samplerate = samplerate
         self.bandwidth = bandwidth
         self.resolution = resolution
         self.latency = latency
         self.window = window
+        self.kernels = kernels
         self.quality = quality
         self.size = size
         self.frequencies = frequencies
@@ -76,10 +85,6 @@ class QDFT:
         self.twiddles = twiddles
         self.inputs = inputs
         self.outputs = outputs
-        self.kernels = kernels
-
-        dfts = numpy.empty((kernels.size, 0, size), dtype=complex)
-        QDFT.accumulate(0, dfts, twiddles, outputs)
 
     def qdft(self, samples):
         """
@@ -101,33 +106,21 @@ class QDFT:
         assert samples.size >= 1
 
         inputs = numpy.concatenate((self.inputs, samples))
-        self.inputs = inputs[samples.size:]
-
         outputs = self.outputs
-
         periods = self.periods
-        offsets = self.offsets + numpy.arange(samples.size)[:, None]
+        offsets = self.offsets
         weights = self.weights
-
         fiddles = self.fiddles
         twiddles = self.twiddles
-
         window = self.window
         kernels = self.kernels
+        size = self.size
 
-        dfts = (fiddles[:, None, None] * inputs[offsets + periods] - inputs[offsets]) * weights
+        numpy.copyto(self.inputs, inputs[samples.size:])
 
-        QDFT.accumulate(samples.size, dfts, twiddles, outputs)
+        dfts = numpy.zeros((samples.size, size, kernels.size), complex)
 
-        numpy.copyto(outputs, dfts[:, -1])
-
-        if window is not None:
-
-            a, b = window[0], window[1] / 2
-
-            return a * dfts[0] + b * (dfts[-1] + dfts[+1])
-
-        return dfts[0]
+        return self.__analyze__(dfts, inputs, outputs, periods, offsets, weights, fiddles, twiddles, window)
 
     def iqdft(self, dfts):
         """
@@ -148,17 +141,76 @@ class QDFT:
         assert dfts.ndim == 2
         assert dfts.size >= 1
 
-        samples = numpy.real(dfts * self.twiddles[0])
+        twiddles = self.twiddles
 
-        return numpy.sum(samples, axis=1)
+        samples = numpy.zeros(len(dfts), float)
 
-    @numba.njit()
-    def accumulate(n, dfts, twiddles, identity):
+        return self.__synthesize__(dfts, samples, twiddles)
 
-        if not n: return
+    @staticmethod
+    @numba.jit(nopython=True, fastmath=True)
+    def __analyze__(dfts, inputs, outputs, periods, offsets, weights, fiddles, twiddles, window):
 
-        dfts[:, 0] = twiddles * (identity + dfts[:, 0])
+        if not dfts.size:
+            return dfts[..., 0]
 
-        for i in range(1, n):
+        dfts[-1] = outputs[:]
 
-            dfts[:, i] = twiddles * (dfts[:, i - 1] + dfts[:, i])
+        if window is not None:
+
+            for i in range(dfts.shape[0]):
+
+                for j in range(dfts.shape[1]):
+
+                    left = inputs[offsets[j] + periods[j] + i]
+                    right = inputs[offsets[j] + i]
+
+                    delta1 = (fiddles[-1] * left - right) * weights[j]
+                    delta2 = (fiddles[ 0] * left - right) * weights[j]
+                    delta3 = (fiddles[+1] * left - right) * weights[j]
+
+                    dfts[i, j, -1] = twiddles[-1, j] * (dfts[i - 1, j, -1] + delta1)
+                    dfts[i, j,  0] = twiddles[ 0, j] * (dfts[i - 1, j,  0] + delta2)
+                    dfts[i, j, +1] = twiddles[+1, j] * (dfts[i - 1, j, +1] + delta3)
+
+        else:
+
+            for i in range(dfts.shape[0]):
+
+                for j in range(dfts.shape[1]):
+
+                    left = inputs[offsets[j] + periods[j] + i]
+                    right = inputs[offsets[j] + i]
+
+                    delta = (fiddles[0] * left - right) * weights[j]
+
+                    dfts[i, j, 0] = twiddles[0, j] * (dfts[i - 1, j, 0] + delta)
+
+        outputs[:] = dfts[-1]
+
+        if window is not None:
+
+            a, b = window[0], window[1] / 2
+
+            for i in range(dfts.shape[0]):
+
+                for j in range(dfts.shape[1]):
+
+                    dfts[i, j, 0] = a * dfts[i, j, 0] + b * (dfts[i, j, -1] + dfts[i, j, +1])
+
+        return dfts[..., 0]
+
+    @staticmethod
+    @numba.jit(nopython=True, fastmath=True)
+    def __synthesize__(dfts, samples, twiddles):
+
+        if not dfts.size:
+            return samples
+
+        for i in range(dfts.shape[0]):
+
+            for j in range(dfts.shape[1]):
+
+                samples[i] += numpy.real(dfts[i, j] * twiddles[0][j])
+
+        return samples
